package pps.core.common.utils;

import com.baomidou.mybatisplus.core.enums.SqlMethod;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfoHelper;
import com.baomidou.mybatisplus.core.toolkit.ReflectionKit;
import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.binding.MapperMethod;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;

import java.io.Serializable;
import java.util.Collection;
import java.util.Date;
import java.util.Objects;

/**
 * 封装工具类
 *
 * @author ZWT
 * @date 2023/08/25 16:08
 */
public class BaseUtils {

    /**
     * 设置实体默认值
     *
     * @param model   实体
     * @param session 会话
     */
    public static void setBaseModelDefault(BaseModel model, PpsUserSession session) {
        String id = model.getId();
        Date now = new Date();
        String userId = session.getId();
        String userName = session.getUserName();
        if (StringUtils.isEmpty(id)) {
            model.setCreateById(userId);
            model.setCreateByName(userName);
            model.setCreateTime(now);
        }
        model.setModifyById(userId);
        model.setModifyByName(userName);
        model.setModifyTime(now);
    }

    /**
     * 批量插入
     *
     * @param mapper      映射器
     * @param entityClass 实体类
     * @param list        列表
     */
    public static <E> void batchInsert(Class<?> mapper, Class<?> entityClass, Collection<E> list) {
        String sqlStatement = SqlHelper.getSqlStatement(mapper, SqlMethod.INSERT_ONE);
        SqlHelper.executeBatch(entityClass, null, list, 1000, (sqlSession, ent) -> {
            sqlSession.insert(sqlStatement, ent);
        });
    }

    /**
     * 批量保存或更新
     *
     * @param baseMapper  基地映射器
     * @param entityClass 实体类
     * @param list        列表
     */
    public static <E> void saveOrUpdateBatch(BaseMapper baseMapper, Class<?> entityClass, Collection<E> list) {
        TableInfo tableInfo = TableInfoHelper.getTableInfo(entityClass);
        String keyProperty = tableInfo.getKeyProperty();
        SqlHelper.executeBatch(entityClass, null, list, 1000, (sqlSession, ent) -> {
            Object idVal = ReflectionKit.getFieldValue(ent, keyProperty);
            if (!com.baomidou.mybatisplus.core.toolkit.StringUtils.checkValNull(idVal) && Objects.nonNull(baseMapper.selectById((Serializable) idVal))) {
                MapperMethod.ParamMap<E> param = new MapperMethod.ParamMap();
                param.put("et", ent);
                sqlSession.update(tableInfo.getSqlStatement(SqlMethod.UPDATE_BY_ID.getMethod()), param);
            } else {
                sqlSession.insert(tableInfo.getSqlStatement(SqlMethod.INSERT_ONE.getMethod()), ent);
            }
        });
    }
}