package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.core.base.constant.ScadaConstant;
import pps.core.base.entity.BaseEnergyStorageDeviceEnt;
import pps.core.base.mapper.BaseEnergyStorageDeviceMapper;
import pps.core.base.service.data.base_energy_storage_device.*;
import pps.core.base.utils.SessionSimulation;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.Date;
import java.util.List;

/**
 * @Author luoxiangyang
 * @Description: 储能设备
 * @Date 2023/8/25
 **/
@XService
public class BaseEnergyStorageDeviceService extends BaseService {
    //todo 模拟登陆
    private final PpsUserSession session = SessionSimulation.getSession();

    @XText("新增")
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBaseEnergyStorageDevice(XContext context, CreateBaseEnergyStorageDeviceInput input) {
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        BaseEnergyStorageDeviceEnt entity = new BaseEnergyStorageDeviceEnt();
        XCopyUtils.copyObject(input, entity);
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getUserName());
        entity.setCreateTime(new Date());
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("更新")
    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBaseEnergyStorageDevice(XContext context, UpdateBaseEnergyStorageDeviceInput input) {
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        QueryWrapper<BaseEnergyStorageDeviceEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .eq(BaseEnergyStorageDeviceEnt::getId, input.getId())
                .eq(BaseEnergyStorageDeviceEnt::getIsDeleted,ScadaConstant.IS_DELETE_FLASE);
        BaseEnergyStorageDeviceEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除")
    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBaseEnergyStorageDevice(XContext context, DeleteBaseEnergyStorageDeviceInput input) {
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        QueryWrapper<BaseEnergyStorageDeviceEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .eq(BaseEnergyStorageDeviceEnt::getId, input.getId())
                .eq(BaseEnergyStorageDeviceEnt::getIsDeleted, ScadaConstant.IS_DELETE_FLASE);
        BaseEnergyStorageDeviceEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        entity = new BaseEnergyStorageDeviceEnt();
        entity.setId(input.getId());
        entity.setIsDeleted(ScadaConstant.IS_DELETE_TRUE);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("根据id获取详情")
    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBaseEnergyStorageDeviceOutput> getBaseEnergyStorageDevice(XContext context, GetBaseEnergyStorageDeviceInput input) {
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        QueryWrapper<BaseEnergyStorageDeviceEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .eq(BaseEnergyStorageDeviceEnt::getId, input.getId())
                .eq(BaseEnergyStorageDeviceEnt::getIsDeleted, ScadaConstant.IS_DELETE_FLASE);
        BaseEnergyStorageDeviceEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseEnergyStorageDeviceOutput output = new GetBaseEnergyStorageDeviceOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("分页查询")
    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseEnergyStorageDeviceOutput> queryBaseEnergyStorageDevice(XContext context, QueryBaseEnergyStorageDeviceInput input) {
        String ouId = input.getOuId();
        String ouName = input.getOuName();
        String deviceName = input.getDeviceName();
        String excludeIdFlag = input.getExcludeIdFlag();
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        List<String> allListByOuId = this.getAllListByOuId(context, input.getOuId());
        QueryWrapper<BaseEnergyStorageDeviceEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .notExists("1".equals(excludeIdFlag), "SELECT 1 FROM base_power_line_storage w WHERE w.ou_id = {0} AND base_energy_storage_device.id = w.storage_id", input.getOuId())
                .eq(BaseEnergyStorageDeviceEnt::getIsDeleted, ScadaConstant.IS_DELETE_FLASE)
                .in(CollUtil.isNotEmpty(allListByOuId), BaseEnergyStorageDeviceEnt::getOuId, allListByOuId)
                .like(StrUtil.isNotEmpty(ouName),BaseEnergyStorageDeviceEnt::getOuName, ouName)
                .like(StrUtil.isNotEmpty(deviceName),BaseEnergyStorageDeviceEnt::getDeviceName, deviceName)
                .orderByDesc(BaseEnergyStorageDeviceEnt::getModifyTime);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseEnergyStorageDeviceOutput.class);
    }
}
