package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.page.PageMethod;
import org.apache.commons.lang3.StringUtils;
import pps.core.base.entity.*;
import pps.core.base.mapper.*;
import pps.core.base.service.data.base_price_strategy.*;
import pps.core.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailOutput;
import pps.core.base.service.data.base_price_strategy_month.CreateBasePriceStrategyMonthInput;
import pps.core.base.service.data.base_price_strategy_month.GetBasePriceStrategyMonthOutput;
import pps.core.base.service.data.base_price_strategy_month.UpdateBasePriceStrategyMonthInput;
import pps.core.base.utils.ServiceUtil;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 市电峰谷配置模块
 *
 * @author ZWT
 * @date 2023/08/24
 */
@XService
@XText("市电峰谷配置模块")
public class BasePriceStrategyService {

    /**
     * 市电峰谷策略--新增
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("市电峰谷策略--新增")
    public XServiceResult createBasePriceStrategy(XContext context, CreateBasePriceStrategyInput input) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        return XTransactionHelper.begin(context, () -> {
            BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
            BasePriceStrategyEnt entity = XCopyUtils.copyNewObject(input, BasePriceStrategyEnt.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.insert(entity);
            String strategyId = entity.getId();
            List<CreateBasePriceStrategyMonthInput> inputMonths = input.getMonths();
            if (CollUtil.isNotEmpty(inputMonths)) {
                List<BasePriceStrategyMonthEnt> monthList = XCopyUtils.copyNewList(inputMonths, BasePriceStrategyMonthEnt.class);
                BasePriceStrategyMonthMapper monthMapper = context.getBean(BasePriceStrategyMonthMapper.class);
                this.saveStrategyMonthDetail(monthMapper, monthList, strategyId, session);
            }
            return XServiceResult.OK;
        });
    }

    /**
     * 市电峰谷策略--修改
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("市电峰谷策略--修改")
    public XServiceResult updateBasePriceStrategy(XContext context, UpdateBasePriceStrategyInput input) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        String strategyId = input.getId();
        return XTransactionHelper.begin(context, () -> {
            BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
            BasePriceStrategyEnt entity = mapper.selectOne(new LambdaQueryWrapper<BasePriceStrategyEnt>()
                    .eq(BaseModel::getId, strategyId));
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            List<UpdateBasePriceStrategyMonthInput> inputMonths = input.getMonths();
            if (CollUtil.isNotEmpty(inputMonths)) {
                List<BasePriceStrategyMonthEnt> monthList = XCopyUtils.copyNewList(inputMonths, BasePriceStrategyMonthEnt.class);
                List<String> collect = monthList.stream()
                        .map(BasePriceStrategyMonthEnt::getStrategyMonth)
                        .collect(Collectors.toList());
                BasePriceStrategyMonthMapper monthMapper = context.getBean(BasePriceStrategyMonthMapper.class);
                //删除月信息
                monthMapper.delete(new QueryWrapper<BasePriceStrategyMonthEnt>()
                        .lambda()
                        .eq(BasePriceStrategyMonthEnt::getStrategyId, strategyId)
                        .in(BasePriceStrategyMonthEnt::getStrategyMonth, collect)
                );
                BasePriceStrategyDetailMapper detailMapper = context.getBean(BasePriceStrategyDetailMapper.class);
                //删除详情
                detailMapper.delete(new QueryWrapper<BasePriceStrategyDetailEnt>()
                        .lambda()
                        .eq(BasePriceStrategyDetailEnt::getStrategyId, strategyId)
                        .in(BasePriceStrategyDetailEnt::getStrategyMonth, collect)
                );
                this.saveStrategyMonthDetail(monthMapper, monthList, strategyId, session);
            }
            XCopyUtils.copyObject(input, entity);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 市电峰谷策略--删除
     * (如果被引用,则不能删除)
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("市电峰谷策略--删除")
    public XServiceResult deleteBasePriceStrategy(XContext context, DeleteBasePriceStrategyInput input) {
        String strategyId = input.getId();
        if (this.checkStrategyIsReference(context, strategyId)) {
            return XServiceResult.error(992, "当前策略已被引用");
        }
        return XTransactionHelper.begin(context, () -> {
            BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
            BasePriceStrategyEnt entity = mapper.selectOne(new LambdaQueryWrapper<BasePriceStrategyEnt>()
                    .eq(BaseModel::getId, strategyId));
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            //删除月数据
            BasePriceStrategyMonthMapper monthMapper = context.getBean(BasePriceStrategyMonthMapper.class);
            monthMapper.delete(new QueryWrapper<BasePriceStrategyMonthEnt>()
                    .lambda()
                    .eq(BasePriceStrategyMonthEnt::getStrategyId, strategyId));
            //删除明细数据
            BasePriceStrategyDetailMapper detailMapper = context.getBean(BasePriceStrategyDetailMapper.class);
            detailMapper.delete(new QueryWrapper<BasePriceStrategyDetailEnt>()
                    .lambda()
                    .eq(BasePriceStrategyDetailEnt::getStrategyId, strategyId));
            mapper.deleteById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 市电峰谷策略--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetBasePriceStrategyOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("市电峰谷策略--详情")
    public XSingleResult<GetBasePriceStrategyOutput> getBasePriceStrategy(XContext context, GetBasePriceStrategyInput input) {
        BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
        String strategyId = input.getId();
        QueryWrapper<BasePriceStrategyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .select(BasePriceStrategyEnt::getId,
                        BasePriceStrategyEnt::getPolicyName,
                        BasePriceStrategyEnt::getAreaCode,
                        BasePriceStrategyEnt::getStrategyYear,
                        BasePriceStrategyEnt::getIsFixedPrice,
                        BasePriceStrategyEnt::getFixedElectricityPrice)
                .eq(BasePriceStrategyEnt::getId, strategyId);
        BasePriceStrategyEnt entity = mapper.selectOne(queryWrapper);
        if (Objects.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePriceStrategyOutput output = XCopyUtils.copyNewObject(entity, GetBasePriceStrategyOutput.class);
        //查月份
        BasePriceStrategyMonthMapper monthMapper = context.getBean(BasePriceStrategyMonthMapper.class);
        List<BasePriceStrategyMonthEnt> monthEntList = monthMapper.selectList(new QueryWrapper<BasePriceStrategyMonthEnt>()
                .lambda()
                .select(BasePriceStrategyMonthEnt::getId, BasePriceStrategyMonthEnt::getStrategyMonth)
                .eq(BasePriceStrategyMonthEnt::getStrategyId, strategyId));
        if (CollUtil.isNotEmpty(monthEntList)) {
            List<GetBasePriceStrategyMonthOutput> months = XCopyUtils.copyNewList(monthEntList, GetBasePriceStrategyMonthOutput.class);
            //查明细
            BasePriceStrategyDetailMapper detailMapper = context.getBean(BasePriceStrategyDetailMapper.class);
            List<BasePriceStrategyDetailEnt> detailEntList = detailMapper.selectList(new QueryWrapper<BasePriceStrategyDetailEnt>()
                    .lambda()
                    .select(BasePriceStrategyDetailEnt::getStrategyMonth,
                            BasePriceStrategyDetailEnt::getPeriodTypeKey,
                            BasePriceStrategyDetailEnt::getStartTime,
                            BasePriceStrategyDetailEnt::getEndTime,
                            BasePriceStrategyDetailEnt::getElectrovalence,
                            BasePriceStrategyDetailEnt::getInternalSettlementPrice)
                    .eq(BasePriceStrategyDetailEnt::getStrategyId, strategyId));
            if (CollUtil.isNotEmpty(detailEntList)) {
                Map<String, String> dictMap = ServiceUtil.getDictMap(context, BusinessConstant.TIME_FRAME);
                List<GetBasePriceStrategyDetailOutput> detailOutputs = XCopyUtils.copyNewList(detailEntList, GetBasePriceStrategyDetailOutput.class);
                for (GetBasePriceStrategyDetailOutput detailOutput : detailOutputs) {
                    detailOutput.setPeriodTypeValue(
                            dictMap.get(detailOutput.getPeriodTypeKey())
                    );
                }
                Map<String, List<GetBasePriceStrategyDetailOutput>> collect = detailOutputs.stream()
                        .collect(Collectors.groupingBy(GetBasePriceStrategyDetailOutput::getStrategyMonth));
                for (GetBasePriceStrategyMonthOutput monthOutput : months) {
                    if (collect.containsKey(monthOutput.getStrategyMonth())) {
                        monthOutput.setDetails(collect.get(monthOutput.getStrategyMonth()));
                    } else {
                        monthOutput.setDetails(new ArrayList<>(0));
                    }
                }
            }
            output.setMonths(months);
        }
        return XSingleResult.success(output);
    }

    /**
     * 市电峰谷策略--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryBasePriceStrategyViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("市电峰谷策略--分页列表")
    public XPageResult<QueryBasePriceStrategyViewOutput> queryBasePriceStrategyView(XContext context, QueryBasePriceStrategyViewInput input) {
        BasePriceStrategyViewMapper mapper = context.getBean(BasePriceStrategyViewMapper.class);
        BasePriceStrategyView strategyView = XCopyUtils.copyNewObject(input, BasePriceStrategyView.class);
        PageMethod.startPage(input.getPage(), input.getLimit());
        List<BasePriceStrategyView> list = mapper.selectList(strategyView);
        PageInfo<BasePriceStrategyView> pageInfo = new PageInfo<>(list);
        List<QueryBasePriceStrategyViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBasePriceStrategyViewOutput.class);
        if (CollUtil.isNotEmpty(outputs)) {
            Map<Integer, String> areaMap = ServiceUtil.getSysAreaMap(context);
            for (QueryBasePriceStrategyViewOutput output : outputs) {
                output.setAreaName(
                        areaMap.get(output.getAreaCode())
                );
            }
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 市电峰谷策略--列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePriceStrategyOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("市电峰谷策略--列表")
    public XListResult<DynamicQueryBasePriceStrategyOutput> queryBasePriceStrategyList(XContext context, DynamicQueryBasePriceStrategyInput input) {
        String id = input.getId();
        String policyName = input.getPolicyName();
        Integer areaCode = input.getAreaCode();
        String strategyYear = input.getStrategyYear();
        Integer isFixedPrice = input.getIsFixedPrice();
        BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
        List<BasePriceStrategyEnt> selectList = mapper.selectList(new LambdaQueryWrapper<BasePriceStrategyEnt>()
                .select(BaseModel::getId, BasePriceStrategyEnt::getPolicyName)
                .eq(StringUtils.isNotBlank(id), BaseModel::getId, id)
                .like(StringUtils.isNotBlank(policyName), BasePriceStrategyEnt::getPolicyName, policyName)
                .eq(Objects.nonNull(areaCode), BasePriceStrategyEnt::getAreaCode, areaCode)
                .eq(Objects.nonNull(strategyYear), BasePriceStrategyEnt::getStrategyYear, strategyYear)
                .eq(Objects.nonNull(isFixedPrice), BasePriceStrategyEnt::getIsFixedPrice, isFixedPrice)
                .orderByDesc(BaseModel::getModifyTime)
        );
        List<DynamicQueryBasePriceStrategyOutput> strategyOutputList = XCopyUtils.copyNewList(selectList, DynamicQueryBasePriceStrategyOutput.class);
        return XListResult.success(strategyOutputList);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 检查市电峰谷策略是否被引用
     *
     * @param context    上下文
     * @param strategyId 战略id
     * @return boolean
     */
    private boolean checkStrategyIsReference(XContext context, String strategyId) {
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        Long count = mapper.selectCount(new LambdaQueryWrapper<BasePowerLineEnt>()
                .eq(BasePowerLineEnt::getStrategyId, strategyId)
        );
        return count > 0;
    }

    /**
     * 保存策略明细
     *
     * @param monthMapper 月映射器
     * @param monthList   月清单
     * @param strategyId  战略id
     * @param session     会话
     */
    private void saveStrategyMonthDetail(BasePriceStrategyMonthMapper monthMapper, List<BasePriceStrategyMonthEnt> monthList, String strategyId, PpsUserSession session) {
        String strategyMonth;
        List<BasePriceStrategyDetailEnt> detailList;
        for (BasePriceStrategyMonthEnt monthEnt : monthList) {
            monthEnt.setStrategyId(strategyId);
            BaseUtils.setBaseModelDefault(monthEnt, session);
            monthMapper.insert(monthEnt);
            strategyMonth = monthEnt.getStrategyMonth();
            detailList = monthEnt.getDetails();
            for (BasePriceStrategyDetailEnt detailEnt : detailList) {
                detailEnt.setStrategyId(strategyId);
                detailEnt.setStrategyMonth(strategyMonth);
                BaseUtils.setBaseModelDefault(detailEnt, session);
            }
            BaseUtils.batchInsert(BasePriceStrategyDetailMapper.class, BasePriceStrategyDetailEnt.class, detailList);
        }
    }
}