package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.core.base.constant.ScadaConstant;
import pps.core.base.entity.BaseWellheadEnt;
import pps.core.base.mapper.BaseWellheadMapper;
import pps.core.base.service.data.base_wellhead.*;
import pps.core.base.utils.SessionSimulation;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.Date;
import java.util.List;

/**
 * @Author luoxiangyang
 * @Description: 井口配置
 * @Date 2023/8/24
 **/
@XService
public class BaseWellheadService extends BaseService {
    //todo 模拟登陆
    private final PpsUserSession session = SessionSimulation.getSession();

    @XText("新增")
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBaseWellhead(XContext context, CreateBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        BaseWellheadEnt entity = new BaseWellheadEnt();
        XCopyUtils.copyObject(input, entity);
        //PpsUserSession session = context.getSession(PpsUserSession.class);
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getUserName());
        entity.setCreateTime(new Date());
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("更新")
    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBaseWellhead(XContext context, UpdateBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        QueryWrapper<BaseWellheadEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseWellheadEnt::getId, input.getId());
        BaseWellheadEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        //PpsUserSession session = context.getSession(PpsUserSession.class);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除")
    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBaseWellhead(XContext context, DeleteBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        QueryWrapper<BaseWellheadEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseWellheadEnt::getId, input.getId());
        BaseWellheadEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        entity = new BaseWellheadEnt();
        entity.setId(input.getId());
        entity.setIsDeleted(ScadaConstant.IS_DELETE_TRUE);
        //PpsUserSession session = context.getSession(PpsUserSession.class);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("根据ID获取详情")
    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBaseWellheadOutput> getBaseWellhead(XContext context, GetBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        QueryWrapper<BaseWellheadEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .eq(BaseWellheadEnt::getId, input.getId())
                .eq(BaseWellheadEnt::getIsDeleted, ScadaConstant.IS_DELETE_FLASE);
        BaseWellheadEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseWellheadOutput output = new GetBaseWellheadOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("分页查询")
    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseWellheadOutput> queryBaseWellhead(XContext context, QueryBaseWellheadInput input) {
        String excludeIdFlag = input.getExcludeIdFlag();
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        QueryWrapper<BaseWellheadEnt> queryWrapper = new QueryWrapper<>();
        List<String> allListByOuId = this.getAllListByOuId(context, input.getOuId());
        queryWrapper.lambda()
                .notExists("1".equals(excludeIdFlag), "SELECT 1 FROM base_power_line_wellhead w WHERE w.ou_id = {0} AND base_wellhead.id = w.wellhead_id", input.getOuId())
                .eq(BaseWellheadEnt::getIsDeleted, ScadaConstant.IS_DELETE_FLASE)
                .in(CollUtil.isNotEmpty(allListByOuId),BaseWellheadEnt::getOuId, allListByOuId)
                .orderByDesc(BaseWellheadEnt::getModifyTime);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseWellheadOutput.class);
    }
}
