package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.*;
import pps.core.space.mapper.*;
import pps.core.space.service.data.space_institution_detail.*;
import pps.core.space.service.data.space_institution_duration.GetSpaceInstitutionDurationViewOutput;
import pps.core.space.service.data.space_institution_wellhead.CreateSpaceInstitutionWellheadInput;
import pps.core.space.service.data.space_institution_wellhead.GetSpaceInstitutionWellheadViewOutput;
import pps.core.space.service.data.space_institution_wellhead.UpdateSpaceInstitutionWellheadInput;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 基础间开配置模块
 *
 * @author ZWT
 * @date 2023/09/05
 */
@XService
@XText("基础间开配置模块")
public class SpaceInstitutionDetailService {

    /**
     * 基础间开配置--新增
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("基础间开配置--新增")
    public XServiceResult createSpaceInstitutionDetail(XContext context, CreateSpaceInstitutionDetailInput input) {
        PpsUserSession session = new PpsUserSession();
        session.setId("123");
        session.setUserName("ceshi");
//        PpsUserSession session = context.getSession(PpsUserSession.class);
        return XTransactionHelper.begin(context, () -> {
            //新增间开配置
            SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
            SpaceInstitutionDetailEnt entity = XCopyUtils.copyNewObject(input, SpaceInstitutionDetailEnt.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.insert(entity);
            String institutionId = entity.getId();
            List<CreateSpaceInstitutionWellheadInput> wellheadList = input.getWellheadList();
            if (CollUtil.isNotEmpty(wellheadList)) {
                SpaceInstitutionWellheadViewMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadViewMapper.class);
                this.saveInstitutionWellhead(session, institutionId, wellheadList, wellheadMapper);
            }
            return XServiceResult.OK;
        });
    }

    /**
     * 基础间开配置--修改
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("基础间开配置--修改")
    public XServiceResult updateSpaceInstitutionDetail(XContext context, UpdateSpaceInstitutionDetailInput input) {
        PpsUserSession session = new PpsUserSession();
        session.setId("123");
        session.setUserName("ceshi");
//        PpsUserSession session = context.getSession(PpsUserSession.class);
        String institutionId = input.getId();
        return XTransactionHelper.begin(context, () -> {
            SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
            SpaceInstitutionDetailEnt entity = this.getInstitutionDetail(mapper, institutionId);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            Integer isCurrentBasic = entity.getIsCurrentBasic();
            if (isCurrentBasic.equals(0)) {
                return XServiceResult.error(992, "基础制度无法修改");
            }
            List<UpdateSpaceInstitutionWellheadInput> wellheadList = input.getWellheadList();
            if (CollUtil.isNotEmpty(wellheadList)) {
                //删除之前关联的信息
                SpaceInstitutionWellheadViewMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadViewMapper.class);
                wellheadMapper.delete(new LambdaQueryWrapper<SpaceInstitutionWellheadView>()
                        .eq(SpaceInstitutionWellheadView::getInstitutionId, institutionId)
                );
                SpaceInstitutionDurationMapper durationMapper = context.getBean(SpaceInstitutionDurationMapper.class);
                durationMapper.delete(new LambdaQueryWrapper<SpaceInstitutionDurationEnt>()
                        .eq(SpaceInstitutionDurationEnt::getInstitutionId, institutionId)
                );
                //重新添加关联信息
                this.saveInstitutionWellhead(session, institutionId, wellheadList, wellheadMapper);
            }
            XCopyUtils.copyObject(input, entity);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 基础间开配置--删除
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("基础间开配置--删除")
    public XServiceResult deleteSpaceInstitutionDetail(XContext context, DeleteSpaceInstitutionDetailInput input) {
        PpsUserSession session = new PpsUserSession();
        session.setId("123");
        session.setUserName("ceshi");
//        PpsUserSession session = context.getSession(PpsUserSession.class);
        String institutionId = input.getId();
        return XTransactionHelper.begin(context, () -> {
            SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
            SpaceInstitutionDetailEnt entity = this.getInstitutionDetail(mapper, institutionId);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            Integer isCurrentBasic = entity.getIsCurrentBasic();
            if (isCurrentBasic.equals(0)) {
                return XServiceResult.error(992, "基础制度无法删除");
            }
            SpaceInstitutionDurationMapper durationMapper = context.getBean(SpaceInstitutionDurationMapper.class);
            durationMapper.update(null, new LambdaUpdateWrapper<SpaceInstitutionDurationEnt>()
                    .eq(SpaceInstitutionDurationEnt::getInstitutionId, institutionId)
                    .set(BaseModel::getIsDeleted, BusinessConstant.DELETE)
                    .set(SpaceInstitutionDurationEnt::getModifyById, session.getId())
                    .set(SpaceInstitutionDurationEnt::getModifyByName, session.getUserName())
                    .set(SpaceInstitutionDurationEnt::getModifyTime, new Date())
            );
            SpaceInstitutionWellheadMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadMapper.class);
            wellheadMapper.update(null, new LambdaUpdateWrapper<SpaceInstitutionWellheadEnt>()
                    .eq(SpaceInstitutionWellheadEnt::getInstitutionId, institutionId)
                    .set(BaseModel::getIsDeleted, BusinessConstant.DELETE)
                    .set(SpaceInstitutionWellheadEnt::getModifyById, session.getId())
                    .set(SpaceInstitutionWellheadEnt::getModifyByName, session.getUserName())
                    .set(SpaceInstitutionWellheadEnt::getModifyTime, new Date())
            );
            mapper.update(null, new LambdaUpdateWrapper<SpaceInstitutionDetailEnt>()
                    .eq(BaseModel::getId, institutionId)
                    .set(BaseModel::getIsDeleted, BusinessConstant.DELETE)
                    .set(SpaceInstitutionDetailEnt::getModifyById, session.getId())
                    .set(SpaceInstitutionDetailEnt::getModifyByName, session.getUserName())
                    .set(SpaceInstitutionDetailEnt::getModifyTime, new Date())
            );
            return XServiceResult.OK;
        });
    }

    /**
     * 基础间开配置--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSpaceInstitutionDetailViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("基础间开配置--详情")
    public XSingleResult<GetSpaceInstitutionDetailViewOutput> getSpaceInstitutionDetailView(XContext context, GetSpaceInstitutionDetailViewInput input) {
        String institutionId = input.getId();
        SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
        SpaceInstitutionDetailEnt entity = this.getInstitutionDetail(mapper, institutionId);
        if (Objects.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSpaceInstitutionDetailViewOutput output = XCopyUtils.copyNewObject(entity, GetSpaceInstitutionDetailViewOutput.class);
        SpaceInstitutionWellheadMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadMapper.class);
        //查井口列表
        List<SpaceInstitutionWellheadEnt> wellheadEntList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionWellheadEnt>()
                .eq(SpaceInstitutionWellheadEnt::getInstitutionId, institutionId)
                .orderByAsc(SpaceInstitutionWellheadEnt::getStartSeq)
        );
        if (CollUtil.isNotEmpty(wellheadEntList)) {
            List<GetSpaceInstitutionWellheadViewOutput> wellheadViewOutputs = XCopyUtils.copyNewList(wellheadEntList, GetSpaceInstitutionWellheadViewOutput.class);
            Map<String, String> runTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.RUN_TYPE);
            Map<String, String> intervalTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.INTERVAL_TYPE);
            wellheadViewOutputs.forEach(wellhead -> {
                wellhead.setRunTypeName(runTypeMap.get(wellhead.getRunTypeKey()));
                wellhead.setIntervalTypeName(intervalTypeMap.get(wellhead.getIntervalTypeKey()));
            });
            //查时段列表
            SpaceInstitutionDurationMapper durationMapper = context.getBean(SpaceInstitutionDurationMapper.class);
            List<SpaceInstitutionDurationEnt> durationEntList = durationMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionDurationEnt>()
                    .eq(SpaceInstitutionDurationEnt::getInstitutionId, institutionId)
                    .orderByAsc(SpaceInstitutionDurationEnt::getOpenWellTime)
            );
            if (CollUtil.isNotEmpty(durationEntList)) {
                List<GetSpaceInstitutionDurationViewOutput> durationViewOutputList = XCopyUtils.copyNewList(durationEntList, GetSpaceInstitutionDurationViewOutput.class);
                Map<String, List<GetSpaceInstitutionDurationViewOutput>> collect = durationViewOutputList.stream()
                        .collect(Collectors.groupingBy(GetSpaceInstitutionDurationViewOutput::getWellheadId));
                wellheadViewOutputs.forEach(wellhead -> {
                    String wellheadId = wellhead.getWellheadId();
                    if (collect.containsKey(wellheadId)) {
                        wellhead.setDurationList(collect.get(wellheadId));
                    } else {
                        wellhead.setDurationList(new ArrayList<>(0));
                    }
                });
            }
            output.setWellheadList(wellheadViewOutputs);
        } else {
            output.setWellheadList(new ArrayList<>(0));
        }
        output.setOuName(ServiceUtil.getSysOrgNameById(context, output.getOuId()));
        output.setGridTypeName(ServiceUtil.getDictMap(context, BusinessConstant.GRID_TYPE).get(output.getGridTypeKey()));
        output.setLineName(ServiceUtil.getPowerLineNameById(context, output.getLineId()));
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySpaceInstitutionDetailOutput> querySpaceInstitutionDetail(XContext context, QuerySpaceInstitutionDetailInput input) {
        SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
        QueryWrapper<SpaceInstitutionDetailEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(SpaceInstitutionDetailEnt::getId, input.getId())
                .eq(SpaceInstitutionDetailEnt::getIsDeleted, input.getIsDeleted())
                .like(SpaceInstitutionDetailEnt::getCreateById, input.getCreateById())
                .like(SpaceInstitutionDetailEnt::getCreateByName, input.getCreateByName())
                .eq(SpaceInstitutionDetailEnt::getCreateTime, input.getCreateTime())
                .like(SpaceInstitutionDetailEnt::getModifyById, input.getModifyById())
                .like(SpaceInstitutionDetailEnt::getModifyByName, input.getModifyByName())
                .eq(SpaceInstitutionDetailEnt::getModifyTime, input.getModifyTime())
                .like(SpaceInstitutionDetailEnt::getOuId, input.getOuId())
                .like(SpaceInstitutionDetailEnt::getLineId, input.getLineId())
                .like(SpaceInstitutionDetailEnt::getGridTypeKey, input.getGridTypeKey())
                .like(SpaceInstitutionDetailEnt::getInstitutionName, input.getInstitutionName())
                .eq(SpaceInstitutionDetailEnt::getInstitutionStartDate, input.getInstitutionStartDate())
                .eq(SpaceInstitutionDetailEnt::getInstitutionEndDate, input.getInstitutionEndDate())
                .eq(SpaceInstitutionDetailEnt::getOptimizeDeadline, input.getOptimizeDeadline())
                .eq(SpaceInstitutionDetailEnt::getOptimizeState, input.getOptimizeState())
                .eq(SpaceInstitutionDetailEnt::getIsCurrentBasic, input.getIsCurrentBasic())
                .eq(SpaceInstitutionDetailEnt::getStartInterval, input.getStartInterval());
        return XMapperHelper.query(mapper, input, queryWrapper, QuerySpaceInstitutionDetailOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySpaceInstitutionDetailOutput> dynamicQuerySpaceInstitutionDetail(XContext context, CustomQueryInput input) {
        SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
        return XMapperHelper.query(mapper, input, SpaceInstitutionDetailEnt.class, QuerySpaceInstitutionDetailOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySpaceInstitutionDetailViewOutput> querySpaceInstitutionDetailView(XContext context, QuerySpaceInstitutionDetailViewInput input) {
        SpaceInstitutionDetailViewMapper mapper = context.getBean(SpaceInstitutionDetailViewMapper.class);
        SpaceInstitutionDetailView record = new SpaceInstitutionDetailView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SpaceInstitutionDetailView> list = mapper.selectList(record);
        PageInfo<SpaceInstitutionDetailView> pageInfo = new PageInfo<>(list);
        List<QuerySpaceInstitutionDetailViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceInstitutionDetailViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 保存策略关联信息
     *
     * @param session        一场
     * @param institutionId  机构id
     * @param wellheadList   井口一览表
     * @param wellheadMapper 井口测绘仪
     */
    private void saveInstitutionWellhead(PpsUserSession session, String institutionId, List wellheadList,
                                         SpaceInstitutionWellheadViewMapper wellheadMapper) {
        List<SpaceInstitutionWellheadView> wellheads = XCopyUtils.copyNewList(wellheadList, SpaceInstitutionWellheadView.class);
        List<SpaceInstitutionDurationEnt> durationList;
        for (SpaceInstitutionWellheadView wellhead : wellheads) {
            wellhead.setInstitutionId(institutionId);
            BaseUtils.setBaseModelDefault(wellhead, session);
            wellheadMapper.insert(wellhead);
            String wellheadId = wellhead.getWellheadId();
            String configId = wellhead.getId();
            durationList = wellhead.getDurationList();
            if (CollUtil.isNotEmpty(durationList)) {
                durationList.forEach(d -> {
                    d.setWellheadId(wellheadId);
                    d.setConfigId(configId);
                    d.setInstitutionId(institutionId);
                    BaseUtils.setBaseModelDefault(d, session);
                });
                BaseUtils.batchInsert(SpaceInstitutionDurationMapper.class, SpaceInstitutionDurationEnt.class, durationList);
            }
        }
    }

    /**
     * 获取间开详情
     *
     * @param mapper        映射器
     * @param institutionId 机构id
     * @return {@link SpaceInstitutionDetailEnt}
     */
    private SpaceInstitutionDetailEnt getInstitutionDetail(SpaceInstitutionDetailMapper mapper, String institutionId) {
        return mapper.selectOne(new LambdaQueryWrapper<SpaceInstitutionDetailEnt>()
                .eq(BaseModel::getId, institutionId)
        );
    }
}
